/*
 * ft245.c  Version 1.01
 *
 * Copyright (c) 2004,2005 Robin Cutshaw	<robin@hamlabs.com>
 *
 * USB driver for FTDI FT-245 based devices
 * including the RFSpace Software Defined Radio - SDR-14
 *
 * This driver is optimized for high speed data transfer from the
 * RFSpace SDR-14 and can be used for other FTDI FT245 based devices.
 * Compile with FT245_SDR14 to work with RFSpace SDR-14 device
 *
 * To create the device inode, use: mknod /dev/usb/rfsdr0 c 180 144
 *
 * ChangeLog:
 *	v1.0  - initial release
 *	v1.01 - prototype cleanup
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


/* Theory of operation
 * 
 * usb_ft245_init() is called when the driver is loaded
 * ft245_probe() is called when a device matching the vendor/product is inserted
 * ft245_open() is called when a user process opens the device for access
 *            the device may only be opened by one process at a time
 *            upon open, a read callback is issued to queue a device read
 * ft245_read_bulk_callback() is called when data is delivered from the device
 *                          data is delivered in the callbackbuf[]
 *                          it is then processed into the protobuf[] based
 *                             on the SDR-14 protocol length bytes
 *                             if SDR14_SPECIFIC is defined
 *                          finally, the data is placed into rbufp[] when
 *                             a complete packet has been processed
 * ft245_read() is called when a user process reads from the device
 *            data is read from the rbufp[]
 * ft245_write() is called when a user process writes to the device
 * ft245_poll() is called when a user process calls select() or poll()
 * ft245_release() is called when a user process closes the device
 * ft245_disconnect() is called when a device is unplugged from the usb bus
 * usb_ft245_cleanup() is called when the driver is unloaded
 *                     it is possible that cleanup is called before
 *                     disconnect()
 * 
 * The FTDI FT245 usb device sends packets of 64 or less bytes at a time
 * with the first two bytes always being modem status bytes.  We ignore
 * these bytes since the 245 has a parallel interface.
 */


#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/random.h>
#include <linux/poll.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/usb.h>

#define FT245_DBG_ERROR		0x01
#define FT245_DBG_INIT		0x02
#define FT245_DBG_PROBE		0x04
#define FT245_DBG_OPENCLOSE	0x08
#define FT245_DBG_POLL		0x10
#define FT245_DBG_WRITE		0x20
#define FT245_DBG_CONTROL	0x40
#define FT245_DBG_READ		0x80
#define FT245_DBG_ALL		0xff

static int debug = 0x07;
#undef dbg
#define dbg(level, format, arg...) do { if (debug&level) printk(KERN_DEBUG __FILE__ ": " format "\n" , ## arg); } while (0)


extern devfs_handle_t usb_devfs_handle;			

#define DRIVER_VERSION "v1.0.1"
#define DRIVER_AUTHOR "Robin Cutshaw, <robin@hamlabs.com>"
#ifdef FT245_SDR14
#define DRIVER_DESC "USB Driver for RFSpace SDR-14 software defined radio"
#else
#define DRIVER_DESC "USB Driver for FTDI FT245 chipset"
#endif

MODULE_PARM(debug, "i");
MODULE_PARM_DESC(debug, "Debug level");

#define	MAX_FT245S		16

#define	USB_FT245_MINOR_BASE	144

#define	READ_BUFFER_SIZE	0x10000		// 64k
#define	MAX_REQ_PACKET_SIZE	0x10000		// 64k
#define	MAX_PROTO_BUF_SIZE	8194		// 8192+2size
#define	RETRY_TIMEOUT		(HZ)
#define	MAX_WRITE_RETRY		5

#define FT245_FLAGS_DEV_OPEN		0x01
#define FT245_FLAGS_RX_BUSY		0x02
#define FT245_FLAGS_INTR_BUSY		0x04
#define FT245_FLAGS_DEV_ERROR		0x08
#define FT245_FLAGS_PENDING_CLEANUP	0x10
#define FT245_FLAGS_MASK			0x1f

#define FTDI_VID		0x0403
#define SDR14_DID		0xf728


static struct usb_device_id ft245_table [] = {
	/* These have the same application level protocol */  
	{ USB_DEVICE(FTDI_VID, SDR14_DID) },	// SDR-14
	{ }					/* Terminating entry */
};

MODULE_DEVICE_TABLE (usb, ft245_table);


struct ft245_state {
	struct usb_device				*dev;		/* USB device handle */
	struct usb_endpoint_descriptor	*endpoint;
	int								inEP;		/* read endpoint */
	int								outEP;		/* write endpoint */
	const struct usb_device_id		*info;		/* SDR14, etc */
	unsigned int					subminor;	/* which minor dev #? */
	struct semaphore				sem;		/* locks this struct */

	char							*rbufp;		/* read buffer for I/O */
	int								rbufhead;
	unsigned int					rbufcnt;

	char							*callbackbuf; /* read buffer for callback */
	int								callbackbufsize;

#ifdef FT245_SDR14
	char							*protobuf;	/* read buffer for callback */
	int								protobufcnt;
	int								protobuffullcnt;
#endif

	char							*wbufp;		/* write buffer for I/O */
	int								wbufsize;

	struct urb						*rx_urb;
	int								flags;
	int								roverruns;
	int								roverruncnt;
	int								usboverruns;

	devfs_handle_t					devfs;		/* devfs device */

	/* always valid */
	wait_queue_head_t				wait;		/* for timed waits */
};


static struct ft245_state *minor_data [MAX_FT245S];

/* make this an rwlock if contention becomes an issue */
static DECLARE_MUTEX (state_table_mutex);

static int		ft245_release(struct inode *, struct file *);
static void		ft245_disconnect(struct usb_device *, void *);
static void		*ft245_probe(struct usb_device *, unsigned int,
	   						const struct usb_device_id *);
static ssize_t	ft245_read(struct file *, char *, size_t, loff_t *);
static ssize_t	ft245_write(struct file *, const char *, size_t, loff_t *);
static int		ft245_open(struct inode *, struct file *);
static unsigned int ft245_poll(struct file *, struct poll_table_struct *);
static void		ft245_dump_buf( char *, char *, int);
static void		ft245_cleanup(struct ft245_state *);
static void 	ft245_read_bulk_callback(struct urb *);
int __init usb_ft245_init(void);
void __exit usb_ft245_cleanup(void);


static /* const */ struct file_operations usb_ft245_fops = {
	owner:		THIS_MODULE,
	read:		ft245_read,
	write:		ft245_write,
	open:		ft245_open,
	release:	ft245_release,
	poll:		ft245_poll,
};


static /* const */ struct usb_driver ft245_driver = {
	name:		"ft245",

	id_table:	ft245_table,
	probe:		ft245_probe,
	disconnect:	ft245_disconnect,

	fops:		&usb_ft245_fops,
	minor:		USB_FT245_MINOR_BASE
};


static void *ft245_probe(struct usb_device *dev, unsigned int ifnum,
	   					const struct usb_device_id *ft245_info)
{
	int				i;
	struct usb_interface_descriptor	*interface;
	struct usb_endpoint_descriptor	*endpoint;
	int				direction, ep;
	char name[8], mfg[30], product[30], serialnumber[30];
	struct ft245_state		*ft245 = NULL;


	if (dev->descriptor.bNumConfigurations != 1
			|| dev->config[0].bNumInterfaces != 1) {
		dbg(FT245_DBG_ERROR, "Bogus ft245 config info");
		return NULL;
	}

	interface = &dev->actconfig->interface[ifnum].altsetting[0];
	if ((interface->bInterfaceClass != USB_CLASS_PER_INTERFACE
		&& interface->bInterfaceClass != USB_CLASS_VENDOR_SPEC)
			|| interface->bNumEndpoints != 2
			) {
		dbg(FT245_DBG_ERROR, "Bogus ft245 interface info");
		dbg(FT245_DBG_ERROR, "biclass %x biend %x",
			interface->bInterfaceClass,
			interface->bNumEndpoints);
		return NULL;
	}
	
	memset(mfg, 0, 30);
	memset(product, 0, 30);
	memset(serialnumber, 0, 30);
	usb_string(dev, dev->descriptor.iManufacturer, mfg, 29);
	usb_string(dev, dev->descriptor.iProduct, product, 29);
	usb_string(dev, dev->descriptor.iSerialNumber, serialnumber, 29);
	dbg(FT245_DBG_PROBE, "ft245_probe mfg/product/sn %s/%s/%s",
		mfg, product, serialnumber);


	down (&state_table_mutex);
	for (i = 0; i < MAX_FT245S; i++) {
		if (!minor_data [i])
			break;
	}
	if (i == MAX_FT245S) {
		info ("Ignoring additional USB FT245s");
		goto bye;
	}

	ft245 = minor_data [i] = kmalloc (sizeof *ft245, GFP_KERNEL);
	if (!ft245) {
		err ("no memory!");
		goto bye;
	}

	init_MUTEX (&ft245->sem);
	ft245->info = ft245_info;
	ft245->subminor = i;

	init_waitqueue_head (&ft245->wait);

	ft245->endpoint = endpoint = interface->endpoint;
	ft245->outEP = ft245->inEP =  -1;

	ep = endpoint [0].bEndpointAddress & USB_ENDPOINT_NUMBER_MASK;
	direction = endpoint [0].bEndpointAddress & USB_ENDPOINT_DIR_MASK;
	if (direction == USB_DIR_IN) {
		ft245->inEP = ep;
		ft245->callbackbufsize = endpoint[0].wMaxPacketSize;
	} else {
		ft245->wbufsize = endpoint[0].wMaxPacketSize;
		ft245->outEP = ep;
	}

	ep = endpoint [1].bEndpointAddress & USB_ENDPOINT_NUMBER_MASK;
	direction = endpoint [1].bEndpointAddress & USB_ENDPOINT_DIR_MASK;
	if (direction == USB_DIR_IN) {
		ft245->inEP = ep;
		ft245->callbackbufsize = endpoint[0].wMaxPacketSize;
	} else {
		ft245->wbufsize = endpoint[0].wMaxPacketSize;
		ft245->outEP = ep;
	}
	dbg(FT245_DBG_PROBE, "max packet size read %d write %d",
		ft245->callbackbufsize, ft245->wbufsize);

	if (!(ft245->rbufp = (char *) kmalloc (READ_BUFFER_SIZE, GFP_KERNEL))) {
		err ("no memory!");
		goto error;
	}
	ft245->rbufhead = 0;
	ft245->rbufcnt = 0;
	// change callbackbufsize from default 64 to 4k
	ft245->callbackbufsize = 4096;
	if (!(ft245->callbackbuf = (char *) kmalloc (ft245->callbackbufsize,
												GFP_KERNEL))) {
		err ("no memory!");
		goto error;
	}
	if (!(ft245->wbufp = (char *) kmalloc (READ_BUFFER_SIZE, GFP_KERNEL))) {
		err ("no memory!");
		goto error;
	}
#ifdef FT245_SDR14
	if (!(ft245->protobuf =
          (char *) kmalloc (MAX_PROTO_BUF_SIZE, GFP_KERNEL))) {
		err ("no memory!");
		goto error;
	}
#endif

	if (ft245->outEP == -1 || ft245->inEP == -1
			|| endpoint [0].bmAttributes != USB_ENDPOINT_XFER_BULK
			|| endpoint [1].bmAttributes != USB_ENDPOINT_XFER_BULK
			) {
		dbg(FT245_DBG_ERROR, "Bogus endpoints");
		goto error;
	}

	info ("USB FT245 #%d connected, major/minor %d/%d", ft245->subminor,
		USB_MAJOR, USB_FT245_MINOR_BASE + ft245->subminor);

	ft245->dev = dev;
	usb_inc_dev_use (dev);

	if ((ft245->rx_urb = usb_alloc_urb(0)) == (struct urb *)NULL) {
		dbg(FT245_DBG_ERROR, "usb_alloc_urb failed");
		goto error;
	}

	/* If we have devfs, register the device */
	sprintf(name, "ft245-%s", serialnumber);
	ft245->devfs = devfs_register(usb_devfs_handle, name,
				       DEVFS_FL_DEFAULT, USB_MAJOR,
				       USB_FT245_MINOR_BASE + ft245->subminor,
				       S_IFCHR | S_IRUSR | S_IWUSR | S_IRGRP |
				       S_IWGRP, &usb_ft245_fops, NULL);

	goto bye;

error:
	minor_data [ft245->subminor] = NULL;
	kfree (ft245);
	ft245 = NULL;
bye:
	up (&state_table_mutex);
	return ft245;
}


static int ft245_open(struct inode *inode, struct file *file)
{
	struct ft245_state	*ft245;
	unsigned int	subminor;
	int			value = 0;
	int			res;
	int			i;
	int			result;

	dbg(FT245_DBG_OPENCLOSE, "ft245_open");
	if (inode == (struct inode *)NULL) {
		dbg(FT245_DBG_ERROR, "ft245_open inode NULL");
		return -ENODEV;
	}
	if (file == (struct file *)NULL) {
		dbg(FT245_DBG_ERROR, "ft245_open file NULL");
		return -ENODEV;
	}
	down (&state_table_mutex);
	subminor = MINOR (inode->i_rdev) - USB_FT245_MINOR_BASE;
	if (subminor >= MAX_FT245S) {
		up (&state_table_mutex);
		return -ENODEV;
	}
	for (i=0; i<MAX_FT245S; i++) {
		ft245 = minor_data [subminor];
		if (ft245 && (ft245->subminor == subminor))
			break;
	}
	if (i == MAX_FT245S) {
		up (&state_table_mutex);
		return -ENODEV;
	}

	down (&ft245->sem);
	up (&state_table_mutex);

	if (ft245->flags & FT245_FLAGS_DEV_OPEN) {
		up (&state_table_mutex);
		value = -EBUSY;
		goto done;
	}

	dbg(FT245_DBG_OPENCLOSE, "open #%d", subminor); 

	file->private_data = ft245;

	ft245->rbufhead = 0;
	ft245->rbufcnt = 0;
	ft245->flags = FT245_FLAGS_DEV_OPEN;
	ft245->roverruns = 0;
	ft245->roverruncnt = 0;
	ft245->usboverruns = 0;
#ifdef FT245_SDR14
	ft245->protobufcnt = 0;
	ft245->protobuffullcnt = 0;
#endif

	FILL_BULK_URB(ft245->rx_urb, ft245->dev,
		usb_rcvbulkpipe(ft245->dev, ft245->inEP),
		ft245->callbackbuf, ft245->callbackbufsize,
		ft245_read_bulk_callback, ft245);
	if ((res = usb_submit_urb(ft245->rx_urb)))
		warn("ft245 failed submit rx_urb %d", res);

	// reset
	result = usb_control_msg(ft245->dev, usb_sndctrlpipe(ft245->dev, 0),
			0, 0x40, 0, 0, NULL, 0, HZ * 5);
	//	request, requesttype, value, index, buffer, length, timeout);
	if (result == -ETIMEDOUT)
		dbg(FT245_DBG_ERROR, "ft245_open control timeout");
	else if (result < 0)
		dbg(FT245_DBG_ERROR, "ft245_open control failed");
	else
		dbg(FT245_DBG_OPENCLOSE, "ft245_open control ok (%d)", result);


done:
	up (&ft245->sem);
	return value;
}


static ssize_t ft245_read(struct file *file, char *buf, size_t len, loff_t *ppos)
{
	struct ft245_state	*ft245;
	int			retval = 0;

	if (len > MAX_REQ_PACKET_SIZE) {
		dbg(FT245_DBG_ERROR, "ft245_read %d too large", len);
		return -EINVAL;
	}

	ft245 = (struct ft245_state *) file->private_data;
	down (&ft245->sem);
	if (!ft245->dev) {
		dbg(FT245_DBG_ERROR, "ft245_read %d no dev!", len);
		retval = -ENODEV;
		goto done;
	}
	if (ft245->flags & FT245_FLAGS_DEV_ERROR) {
		dbg(FT245_DBG_ERROR, "ft245_read device error");
		retval = -EIO;
		goto done;
	}
	dbg(FT245_DBG_READ, "ft245_read ask %d avail %d head %d flags 0x%x",
		len, ft245->rbufcnt, ft245->rbufhead, ft245->flags);

	if (!(ft245->flags & FT245_FLAGS_DEV_OPEN)) {
		dbg(FT245_DBG_ERROR, "ft245_read not open! (flags 0x%x) 1",
			ft245->flags);
		retval = -ENODEV;
		goto done;
	}


	if ((ft245->flags & FT245_FLAGS_RX_BUSY) || (ft245->rbufcnt == 0)) {
		if (file->f_flags & O_NONBLOCK) {
			retval = -EAGAIN;
			goto done;
		}

		while ((ft245->flags & FT245_FLAGS_RX_BUSY) || (ft245->rbufcnt == 0)) {
			if (signal_pending(current)) {
				retval = -EINTR;
				goto done;
			}
			up (&ft245->sem);
			interruptible_sleep_on(&ft245->wait);
			down(&ft245->sem);
			if (!(ft245->flags & FT245_FLAGS_DEV_OPEN))
				break;
		}
	}

	if (!(ft245->flags & FT245_FLAGS_DEV_OPEN)) {
		dbg(FT245_DBG_ERROR, "ft245_read not open! (flags 0x%x) 2",
			ft245->flags);
		retval = -ENODEV;
		goto done;
	}

	if (!ft245->dev) {
		dbg(FT245_DBG_ERROR, "ft245_read no dev!");
		retval = -ENODEV;
		goto done;
	}

	if (ft245->rbufcnt < len)
		len = ft245->rbufcnt;
	if (copy_to_user (buf, &ft245->rbufp[ft245->rbufhead], len)) {
		retval = -EFAULT;
		goto done;
	} else {
		dbg(FT245_DBG_READ, "ft245_read returned %d", len);
		retval = len;
		ft245->rbufcnt -= len;
		if (ft245->rbufcnt == 0)
			ft245->rbufhead = 0;
		else
			ft245->rbufhead += len;
	}

done:
	up (&ft245->sem);
	return retval;
}


static ssize_t ft245_write(struct file *file,
						const char *buf, size_t len, loff_t *ppos)
{
	struct ft245_state	*ft245;
	ssize_t			bytes_written = 0;

	if (len > MAX_REQ_PACKET_SIZE) {
		dbg(FT245_DBG_ERROR, "ft245_write %d too large", len);
		return -EINVAL;
	}

	ft245 = (struct ft245_state *) file->private_data;
	down (&ft245->sem);
	if (!ft245->dev) {
		up (&ft245->sem);
		dbg(FT245_DBG_ERROR, "ft245_write %d no dev!", len);
		return -ENODEV;
	}
	if (!(ft245->flags & FT245_FLAGS_DEV_OPEN)) {
		dbg(FT245_DBG_ERROR, "ft245_write %d not open!", len);
		up (&ft245->sem);
		return -ENODEV;
	}
	if (ft245->flags & FT245_FLAGS_DEV_ERROR) {
		dbg(FT245_DBG_ERROR, "ft245_write device error");
		return -EIO;
	}

	dbg(FT245_DBG_WRITE, "ft245_write len %d flags 0x%x", len, ft245->flags);
	
	while (len > 0) {
		char		*obuf = ft245->wbufp;
		int		maxretry = MAX_WRITE_RETRY;
		unsigned long	copy_size, thistime;

		thistime = copy_size = len;
		if (copy_from_user (obuf, buf, copy_size)) {
			bytes_written = -EFAULT;
			break;
		}
		ft245_dump_buf("write ", obuf, copy_size);
		while (thistime) {
			int		result;
			unsigned int		count;

			if (signal_pending (current)) {
				if (!bytes_written)
					bytes_written = -EINTR;
				goto done;
			}

			result = usb_bulk_msg (ft245->dev,
				 usb_sndbulkpipe (ft245->dev, ft245->outEP),
				 obuf, thistime, (int*)&count, HZ*10);

			if (count != thistime)
				dbg(FT245_DBG_WRITE, "write USB count %d thistime %d",
					(int )count, (int )thistime);
			if (result)
				dbg(FT245_DBG_WRITE, "write USB err - %d", result);

			if (count) {
				obuf += count;
				thistime -= count;
				maxretry = MAX_WRITE_RETRY;
				continue;
			} else if (!result)
				break;
				
			if (result == USB_ST_TIMEOUT) {	/* NAK - delay a bit */
				if (!maxretry--) {
					if (!bytes_written)
						bytes_written = -ETIME;
					goto done;
				}
                                interruptible_sleep_on_timeout (&ft245->wait,
					RETRY_TIMEOUT);
				continue;
			} 
			if (!bytes_written)
				bytes_written = -EIO;
			goto done;
		}
		bytes_written += copy_size;
		len -= copy_size;
		buf += copy_size;
	}
done:
	up (&ft245->sem);
	dbg(FT245_DBG_WRITE, "wrote %Zd", bytes_written); 
	return bytes_written;
}


static void ft245_read_bulk_callback(struct urb *urb)
{
	struct ft245_state *ft245 = urb->context;
	int  res, rbuftail, callbackndx, chunksize;
	int count = urb->actual_length;
#ifdef FT245_SDR14
	int  movesize;
#endif

	if (!ft245) {
		dbg(FT245_DBG_ERROR, "ft245_read_callback ft245 NULL!");
		return;
	}
	if (urb->status != USB_ST_NOERROR)
		dbg(FT245_DBG_READ, "ft245_read_bulk_callback count %d flags %d status %d over %d overcnt %d usbover %d",
			count, ft245->flags, urb->status,
			ft245->roverruns, ft245->roverruncnt, ft245->usboverruns);

	if (!(ft245->flags & FT245_FLAGS_DEV_OPEN)) {
		dbg(FT245_DBG_READ, "ft245_read_bulk_callback dev not open!");
		return;
	}
	if (ft245->flags & FT245_FLAGS_RX_BUSY) {
		dbg(FT245_DBG_ERROR, "ft245 RX busy");
		goto reload;
	}
	ft245->flags |= FT245_FLAGS_RX_BUSY;

	switch (urb->status) {
		case USB_ST_DATAOVERRUN:
			ft245->usboverruns++;
			dbg(FT245_DBG_ERROR, "read data overrun");
			break;
		case USB_ST_NOERROR:
			// valid data is available
			if (count > 2) {
#ifndef FT245_SDR14
				dbg(FT245_DBG_CONTROL,
				"read count %d head/cnt %d/%d (%d+%d) %02x %02x",
					count, ft245->rbufhead, ft245->rbufcnt, count/64, count%64,
					(unsigned char)ft245->callbackbuf[2],
					(unsigned char )ft245->callbackbuf[3]);
#else
				dbg(FT245_DBG_CONTROL,
				"read count %d head/cnt %d/%d (%d+%d) %02x %02x full/cnt %d/%d",
					count, ft245->rbufhead, ft245->rbufcnt, count/64, count%64,
					(unsigned char)ft245->callbackbuf[2],
					(unsigned char )ft245->callbackbuf[3],
					ft245->protobuffullcnt, ft245->protobufcnt);
#endif
if (count < 32)	ft245_dump_buf("read ", 
                        ft245->callbackbuf, count < 16 ? count : 16);
			}
			callbackndx = 0;
			while (callbackndx < count) {
				chunksize = count - callbackndx;
				if (chunksize > 64)
					chunksize = 64;
				if (chunksize < 2) {
					dbg(FT245_DBG_ERROR, "ft245_read_bulk_callback short %d",
							chunksize);
					ft245_dump_buf("read ", &ft245->callbackbuf[callbackndx],
							chunksize);
					break;
				}
				if ((ft245->callbackbuf[callbackndx+0] != 0x31) ||
				    ((ft245->callbackbuf[callbackndx+1] != 0x60) &&
				     (ft245->callbackbuf[callbackndx+1] != 0x00))) {
					dbg(FT245_DBG_ERROR,
						"2 byte header error!  May be out of sync.");
					dbg(FT245_DBG_ERROR, "ft245_read_bulk_callback count %d chunksize %d flags %d status %d over %d overcnt %d usbover %d",
						count, chunksize, ft245->flags,
						urb->status, ft245->roverruns,
						ft245->roverruncnt,
						ft245->usboverruns);
					ft245_dump_buf("readerr ",
						&ft245->callbackbuf[callbackndx],
						chunksize < 16 ? chunksize :16);
					break;
				}
				if (chunksize == 2)
					break;
				// skip the two status bytes from the FTDI chip
				callbackndx += 2;
				chunksize -= 2;
				// we now have between 2 and 62 bytes of data
#ifndef FT245_SDR14
				rbuftail = ft245->rbufhead + ft245->rbufcnt;
				if (rbuftail + chunksize > READ_BUFFER_SIZE) {
					ft245->roverruns++;
					ft245->roverruncnt += chunksize;
					dbg(FT245_DBG_ERROR,
						"ft245_read_bulk_callback dropped %d", chunksize);
				} else {
					memcpy(&ft245->rbufp[rbuftail],
						&ft245->callbackbuf[callbackndx],
						chunksize);
					ft245->rbufcnt += chunksize;
					dbg(FT245_DBG_CONTROL,
					"ft245_read_bulk_callback added %d", chunksize);
					ft245_dump_buf("read chunk ", &ft245->callbackbuf[callbackndx],
						chunksize < 16 ?  chunksize : 16);
					wake_up_interruptible(&ft245->wait);
				}
				callbackndx += chunksize;
#else
				while (chunksize > 0) {
					if (ft245->protobuffullcnt == 0) {
						if (ft245->protobufcnt == 0) {
							ft245->protobuf[0] = ft245->callbackbuf[callbackndx];
							ft245->protobufcnt = 1;
							callbackndx++;
							chunksize--;
							continue;
						}
						if (ft245->protobufcnt == 1) {
							ft245->protobuf[1] = ft245->callbackbuf[callbackndx];
							ft245->protobufcnt = 2;
							callbackndx++;
							chunksize--;
							ft245->protobuffullcnt =
								(unsigned int)(ft245->protobuf[1]&0x1f)*256 +
						   	        (unsigned int)(ft245->protobuf[0]);
							if (ft245->protobuffullcnt == 0)
								ft245->protobuffullcnt = 8194;
							dbg(FT245_DBG_CONTROL, "proto packet size %d",
								ft245->protobuffullcnt);
						}
						else {
							// shouldn't happen!
							dbg(FT245_DBG_ERROR, "no protofull with count %d",
								ft245->protobufcnt);
							break;
						}
					}
					if ((ft245->protobufcnt + chunksize) >
                        ft245->protobuffullcnt) {
						movesize = ft245->protobuffullcnt - ft245->protobufcnt;
						dbg(FT245_DBG_CONTROL, "chunksize split (%d/%d/%d/%d)",
							chunksize, ft245->protobufcnt, movesize,
							(ft245->protobufcnt + chunksize) -
							ft245->protobuffullcnt);
						memcpy(&ft245->protobuf[ft245->protobufcnt],
							&ft245->callbackbuf[callbackndx], movesize);
						callbackndx += movesize;
						chunksize -= movesize;
						ft245->protobufcnt = ft245->protobuffullcnt;
					} else {
						memcpy(&ft245->protobuf[ft245->protobufcnt],
							&ft245->callbackbuf[callbackndx],
							chunksize);
						ft245->protobufcnt += chunksize;
						callbackndx += chunksize;
						chunksize = 0;
					}
					if (ft245->protobufcnt == ft245->protobuffullcnt) {
						rbuftail = ft245->rbufhead + ft245->rbufcnt;
						// check for overflow
						if (rbuftail + ft245->protobufcnt > READ_BUFFER_SIZE) {
							ft245->roverruns++;
							ft245->roverruncnt += ft245->protobufcnt;
							dbg(FT245_DBG_ERROR,
								"ft245_read_bulk_callback dropped %d",
								ft245->protobufcnt);
						} else {
							memcpy(&ft245->rbufp[rbuftail],
								ft245->protobuf,
								ft245->protobuffullcnt);
							ft245->rbufcnt += ft245->protobuffullcnt;
							dbg(FT245_DBG_CONTROL,
							"ft245_read_bulk_callback added full protobuf %d",
								ft245->protobufcnt);
							ft245_dump_buf("read chunk ", ft245->protobuf,
								ft245->protobuffullcnt < 16 ?
								ft245->protobuffullcnt : 16);
							wake_up_interruptible(&ft245->wait);
						}
						ft245->protobufcnt = 0;
						ft245->protobuffullcnt = 0;
					}
				}
#endif
			}
			goto reload;
		case USB_ST_NORESPONSE:
			dbg(FT245_DBG_ERROR, "read no response");
			break;
		case USB_ST_URB_KILLED:
			dbg(FT245_DBG_ERROR, "read urb killed");
			break;
		case -ECONNRESET:
			dbg(FT245_DBG_ERROR, "read connection reset");
			break;
		case USB_ST_CRC:
			dbg(FT245_DBG_ERROR, "CRC error");
			break;
		case USB_ST_INTERNALERROR:
			dbg(FT245_DBG_ERROR, "internal error");
			break;
		default:
			dbg(FT245_DBG_ERROR, "ft245 RX status %d", urb->status);
			break;
	}

	ft245->flags &= ~FT245_FLAGS_RX_BUSY;
	ft245->flags |= FT245_FLAGS_DEV_ERROR;
	wake_up_interruptible(&ft245->wait);
	return;

reload:
	FILL_BULK_URB(ft245->rx_urb, ft245->dev,
		usb_rcvbulkpipe(ft245->dev, ft245->inEP),
		ft245->callbackbuf, ft245->callbackbufsize,
		ft245_read_bulk_callback, ft245);
	//ft245->rx_urb->transfer_flags |= USB_ASYNC_UNLINK;
	ft245->rx_urb->status = 0;
	if ((res = usb_submit_urb(ft245->rx_urb)))
		warn("ft245 failed submit rx_urb %d", res);
	ft245->flags &= ~FT245_FLAGS_RX_BUSY;
	wake_up_interruptible(&ft245->wait);
}


static unsigned int ft245_poll(struct file *file, struct poll_table_struct *wait)
{
	struct ft245_state	*ft245;
	unsigned int		rp, wp;
	static unsigned int		lastrp = 0, lastwp = 0;

	ft245 = (struct ft245_state *) file->private_data;
	if (!ft245 || !ft245->dev) {
		dbg(FT245_DBG_ERROR, "ft245_poll NULL ft245!");
		return -ENODEV;
	}
	if (!(ft245->flags & FT245_FLAGS_DEV_OPEN)) {
		dbg(FT245_DBG_ERROR, "ft245_poll device not open");
		return POLLERR | POLLHUP;
	}
	if (ft245->flags & FT245_FLAGS_DEV_ERROR) {
		dbg(FT245_DBG_ERROR, "ft245_poll device error");
		return POLLERR | POLLHUP;
	}
	poll_wait(file, &ft245->wait, wait);
	if (!ft245 || !ft245->dev) {
		dbg(FT245_DBG_ERROR, "ft245_poll NULL ft245! (after wait)");
		return -ENODEV;
	}
	if (!(ft245->flags & FT245_FLAGS_DEV_OPEN)) {
		dbg(FT245_DBG_ERROR, "ft245_poll device not open (after wait)");
		return POLLERR | POLLHUP;
	}
	if (ft245->flags & FT245_FLAGS_DEV_ERROR) {
		dbg(FT245_DBG_ERROR, "ft245_poll device error");
		return POLLERR | POLLHUP;
	}
	rp = ft245->rbufcnt == 0 ? 0 : POLLIN | POLLRDNORM;
	wp = POLLOUT | POLLWRNORM;
	if ((lastrp != rp) || (lastwp != wp)) {
		dbg(FT245_DBG_POLL, "ft245_poll rd 0x%02x wr 0x%02x", rp, wp);
		lastrp = rp;
		lastwp = wp;
	}

	return rp | wp;
}


static int ft245_release(struct inode *inode, struct file *file)
{
	struct ft245_state	*ft245;

	dbg(FT245_DBG_OPENCLOSE, "ft245_release");
	if (!file) {
		dbg(FT245_DBG_ERROR, "Release with no file structure!");
		return 0;
	}
	ft245 = (struct ft245_state *) file->private_data;
	if (!ft245) {
		dbg(FT245_DBG_ERROR, "Release with no device structure!");
		return 0;
	}
	if (ft245->flags & FT245_FLAGS_PENDING_CLEANUP)
		dbg(FT245_DBG_ERROR, "Release with device pending cleanup!");
	else if (!(ft245->flags & FT245_FLAGS_DEV_OPEN)) {
		dbg(FT245_DBG_ERROR, "Release with device not open (flags)!");
		return 0;
	}
	if (ft245->subminor != (MINOR (inode->i_rdev) - USB_FT245_MINOR_BASE)) {
		dbg(FT245_DBG_ERROR, "Release with subminor not correct!");
		return 0;
	}
	if ((ft245->flags & ~FT245_FLAGS_MASK) != 0) {
		dbg(FT245_DBG_ERROR, "Release corrupt flags! 0x%x", ft245->flags);
		return 0;
	}
	dbg(FT245_DBG_OPENCLOSE, "ft245_release flags 0x%x close #%d intr cnt %d over %d overcnt %d usbover %d",
		ft245->flags, ft245->subminor, 0 /*ft245->intrcnt*/, ft245->roverruns,
		ft245->roverruncnt, ft245->usboverruns); 

	down (&state_table_mutex);
	down (&ft245->sem);

	if (ft245->flags & FT245_FLAGS_PENDING_CLEANUP) {
		ft245_cleanup(ft245);
	} else {
		ft245->flags = 0;
		usb_unlink_urb(ft245->rx_urb);
		up (&ft245->sem);
	}
	
	up (&state_table_mutex);

	return 0;
}


static void ft245_disconnect(struct usb_device *dev, void *ptr)
{
	struct ft245_state	*ft245 = (struct ft245_state *) ptr;
	int			subminor = ft245->subminor;
	int			i;

	dbg(FT245_DBG_OPENCLOSE, "ft245_disconnect");
	if (!ft245) {
		dbg(FT245_DBG_ERROR, "ft245_disconnect NULL ft245!");
		return;
	}
	dbg(FT245_DBG_OPENCLOSE, "ft245_disconnect flags 0x%x", ft245->flags);
	down (&state_table_mutex);
	down (&ft245->sem);

	for (i=0; i<MAX_FT245S; i++)
		if (minor_data[i] == ft245) {
			minor_data [i] = NULL;
			break;
		}
	if (i == MAX_FT245S)
		dbg(FT245_DBG_ERROR, "disconnect ft245 not found!");


	if (ft245->flags & FT245_FLAGS_DEV_OPEN) {
		// disconnect before release!
		dbg(FT245_DBG_ERROR, "ft245_disconnect Disconnect before release!");
		usb_unlink_urb(ft245->rx_urb);
		ft245->flags = FT245_FLAGS_PENDING_CLEANUP;
		up(&ft245->sem);
	} else {
		ft245->flags = 0;
		ft245_cleanup(ft245);
	}

	info ("USB FT245 #%d disconnected", subminor);
	usb_dec_dev_use (dev);

	up (&state_table_mutex);
}


static void ft245_cleanup(struct ft245_state *ft245)
{
	usb_free_urb(ft245->rx_urb);
	wake_up_interruptible(&ft245->wait);
	devfs_unregister(ft245->devfs); 
	kfree (ft245->rbufp);
	kfree (ft245->callbackbuf);
	kfree (ft245->wbufp);
	// down was called before this
	up (&ft245->sem);
	kfree (ft245);
}


static void ft245_dump_buf( char *str, char *buf, int len)
{
	int i, j, linendx;
	unsigned char linebuf[81], c;

	if (!buf)
		return;

	for (i = 0; i < len; i += 16) {
		linendx = 0;
		for (j = 0; (j < 16) && (i+j < len); j++) {
			c = buf[i+j];
			if (((c>>4) & 0x0f) > 9)
				linebuf[linendx++] = ((c>>4) & 0x0f)-10 + 'a';
			else
				linebuf[linendx++] = ((c>>4) & 0x0f) + '0';
			if ((c & 0x0f) > 9)
				linebuf[linendx++] = (c & 0x0f)-10 + 'a';
			else
				linebuf[linendx++] = (c & 0x0f) + '0';
			linebuf[linendx++] = ' ';
		}
		for (; j < 16; j++) {
			linebuf[linendx++] = ' ';
			linebuf[linendx++] = ' ';
			linebuf[linendx++] = ' ';
		}
		linebuf[linendx++] = ' ';
		linebuf[linendx++] = '|';
		for (j = 0; (j < 16) && (i+j < len); j++) {
			c = buf[i+j];
			if ((c >= '!') && (c <= '~'))
				linebuf[linendx++] = c;
			else
				linebuf[linendx++] = '.';
		}
		for (; j < 16; j++)
			linebuf[linendx++] = ' ';
		linebuf[linendx++] = '|';
		linebuf[linendx++] = '\0';
		dbg((FT245_DBG_WRITE|FT245_DBG_CONTROL|FT245_DBG_READ), "%s%s", str, linebuf);
	}
}


int __init usb_ft245_init(void)
{
	dbg(FT245_DBG_INIT, "FT245 driver init (debug=0x%02x)", debug);
 	if (usb_register (&ft245_driver) < 0)
 		return -1;
	info(DRIVER_VERSION ":" DRIVER_DESC);
	return 0;
}


void __exit usb_ft245_cleanup(void)
{
	dbg(FT245_DBG_INIT, "FT245 driver exit");
	usb_deregister (&ft245_driver);
}


module_init (usb_ft245_init);
module_exit (usb_ft245_cleanup);

MODULE_AUTHOR( DRIVER_AUTHOR );
MODULE_DESCRIPTION( DRIVER_DESC );
// The statement MODULE_LICENSE("GPL"); causes a fatal error
// on old 2.4 kernels (e.g. Mandrake 8.0) and is therefore
// commented out.
// On recent kernels (e.g. RedHat 9) a missing licence causes
// an error message about "tainted kernel" but that is not fatal.
// XXXXXXXXXXXXXXXXXXX not always defined: MODULE_LICENSE("GPL");

